﻿------------------------------------------------------------------------------------------
--Setup the Class Power-------------------------------------------------------------------
------------------------------------------------------------------------------------------
local SN, L, F, T, M, _ = unpack(select(2,...))

local UnitHasVehicleUI = UnitHasVehicleUI
local unpack = unpack

SN["classMaxResourceBar"] = {
	["DEATHKNIGHT"] = 6,
	["PALADIN"] = 5,
	["WARLOCK"] = 4,
	["PRIEST"] = 3,
	["MONK"] = 5,
	["MAGE"] = 6,
}

function SN:ClassPower()
	F.classpower = CreateFrame("Frame", nil, UIParent)
	F.classpower:SetPoint(unpack(SN.data.profile.frames.classpower))
	F.classpower:SetSize(SN.data.profile.classResources.width, SN.data.profile.classResources.height)
	F.classpower.frame = nil
	F.classpower.name = "Class Power"
	F.classpower.value = F.classpower:CreateFontString(nil, "OVERLAY")
	F.classpower.value:SetPoint("CENTER", F.classpower, "CENTER", 4, 0)
	F.classpower.value:SetFontObject(GameFontNormal)
	F.classpower.value:SetJustifyH("CENTER")
	F.classpower.value:SetShadowOffset(1, -1)
	
	F.classpower:RegisterEvent("PLAYER_ENTERING_WORLD")
	F.classpower:RegisterEvent("UNIT_DISPLAYPOWER")
		
	F.classpower:SetScript("OnEvent", function(self, event, ...)
		if event == "PLAYER_ENTERING_WORLD" then
			SN:SetupRunes()
			SN:SetupCombo()
			SN:SetupHoly()
			SN:SetupEclipse()
			SN:SetupChi()

			local class = select(2, UnitClass("player"))
			if SN.data.profile.classResources.enable then
				if class == "PALADIN" then
					PaladinPowerBar:Hide()
					PaladinPowerBar:UnregisterAllEvents()
				end

				if class == "DEATHKNIGHT" then
					RuneFrame:Hide()
					RuneFrame:UnregisterAllEvents()
				end

				if class == "ROGUE" or "DRUID" then	
					ComboFrame:Hide()
					ComboFrame:UnregisterAllEvents()
				end

				if class == "DRUID" then
					EclipseBarFrame:Hide()
					EclipseBarFrame:UnregisterAllEvents()
				end

				if class == "MONK" then
					MonkHarmonyBar:Hide()
					MonkHarmonyBar:UnregisterAllEvents()
				end
			end
		end

		if event == "UNIT_DISPLAYPOWER" then
			if self.frame ~= nil then
				if UnitHasVehicleUI("player") then
					self.frame:Hide()
				else
					self.frame:Show()
				end
			end
		end
	end)

	if SN.data.profile.classResources.enable == false then
		SN:ClassDisable()
	else
		if (IsAddOnLoaded("ElvUI")) then
			SN:ElvOverride()
		end
	end
end

function SN:ClassResize()
	local class = select(2, UnitClass("player"))
	local BORDER = 2
	local MAX_HOLY_POWER = 5
	local MAX_COMBO_POINTS = MAX_COMBO_POINTS
	local CLASSBAR_WIDTH = F.classpower:GetWidth() - (BORDER*2)
	local maxRune = SN['classMaxResourceBar']["DEATHKNIGHT"]
	local maxChi = SN['classMaxResourceBar']["MONK"]
	
	F.classpower:SetSize(SN.data.profile.classResources.width, SN.data.profile.classResources.height)

	if class == "DEATHKNIGHT" then
		F.classpower.runes:SetSize(CLASSBAR_WIDTH, F.classpower:GetHeight())
		for i = 1, maxRune do
			F.classpower.runes[i]:SetSize((F.classpower.runes:GetWidth() - 15) / 6, F.classpower.runes:GetHeight())
		end
	elseif class == "PALADIN" then
		F.classpower.holypower:SetSize(CLASSBAR_WIDTH, F.classpower:GetHeight())
		for i = 1, MAX_HOLY_POWER do
			F.classpower.holypower[i]:SetSize((F.classpower.holypower:GetWidth() - 16) / MAX_HOLY_POWER, F.classpower.holypower:GetHeight())
		end
	elseif class == "DRUID" or class == "ROGUE" then
		if class == "DRUID" and GetSpecialization() == 1 then
			F.classpower.eclipse:SetSize(F.classpower:GetWidth(), F.classpower:GetHeight())
			F.classpower.eclipse.lunar:SetSize(F.classpower.eclipse:GetWidth() - 1, F.classpower.eclipse:GetHeight() - 1)
			F.classpower.eclipse.solar:SetSize(F.classpower.eclipse:GetWidth() - 1, F.classpower.eclipse:GetHeight() - 1)
		end
		F.classpower.points:SetSize(CLASSBAR_WIDTH, F.classpower:GetHeight())
		for i = 1, MAX_COMBO_POINTS do
			F.classpower.points[i]:SetSize((F.classpower.points:GetWidth() - 16) / MAX_COMBO_POINTS, F.classpower.points:GetHeight())
		end
	elseif class == "MONK" then
		F.classpower.chi:SetSize(CLASSBAR_WIDTH, F.classpower:GetHeight())
		local curChi = UnitPowerMax("player", SPELL_POWER_CHI)

		for i = 1, curChi do
			F.classpower.chi[i]:SetSize((F.classpower.chi:GetWidth() - 16) / curChi, F.classpower.chi:GetHeight())
		end
	end
end	

function SN:BlizzClassPowerPower(info, value)
	local class = select(2, UnitClass("player"))

	if class == "PALADIN" then
		local visible = PaladinPowerBar:IsVisible()
		if SN.data.profile.classResources[info[#info]] then
			if visible then
				PaladinPowerBar:Hide()
				PaladinPowerBar:UnregisterAllEvents()
			end
		elseif SN.data.profile.classResources[info[#info]] == false then
			if not visible then
				PaladinPowerBar:Show()
				PaladinPowerBar:GetScript("OnLoad")(PaladinPowerBar)
			end
		end
    end

	if class == "DEATHKNIGHT" then
		local visible = RuneFrame:IsVisible()
		if SN.data.profile.classResources[info[#info]] then
			if visible then
				RuneFrame:Hide()
				RuneFrame:UnregisterAllEvents()
			end
		elseif SN.data.profile.classResources[info[#info]] == false then
			if not visible then
				RuneFrame:Show()
				RuneFrame:GetScript("OnLoad")(RuneFrame)
			end
		end
	end

	if class == "ROGUE" or "DRUID" then
		local visible = ComboFrame:IsVisible()
		if SN.data.profile.classResources[info[#info]] then
			if visible then
				ComboFrame:Hide()
				ComboFrame:UnregisterAllEvents()
			end
		elseif SN.data.profile.classResources[info[#info]] == false then
			if not visible then
				ComboFrame:Show()
				ComboFrame:GetScript("OnLoad")(ComboFrame)
			end
		end
	end

	if class == "DRUID" then
		local spec = GetSpecialization()
		local visible = EclipseBarFrame:IsVisible()
		if SN.data.profile.classResources[info[#info]] and spec == 1 then
			if visible then
				EclipseBarFrame:Hide()
				EclipseBarFrame:UnregisterAllEvents()
			end
		elseif SN.data.profile.classResources[info[#info]] == false and spec == 1 then
			if not visible then
				EclipseBarFrame:Show()
				EclipseBarFrame:GetScript("OnLoad")(EclipseBarFrame)
			end
		end
	end

	if class == "MONK" then
		local visible = MonkHarmonyBar:IsVisible()
		if SN.data.profile.classResources[info[#info]] then
			if visible then
				MonkHarmonyBar:Hide()
				MonkHarmonyBar:UnregisterAllEvents()
			end
		elseif SN.data.profile.classResources[info[#info]] == false then
			if not visible then
				MonkHarmonyBar:Show()
				MonkHarmonyBar:GetScript("OnLoad")(MonkHarmonyBar)
			end
		end
	end
end

function SN:ClassEnable()
	F.classpower:Show()
end

function SN:ClassDisable()
	F.classpower:Hide()
end