------------------------------------------------------------------------------------------
--Power Bar Setup-------------------------------------------------------------------------
------------------------------------------------------------------------------------------
local SN, L, F, T, M, _ = unpack(select(2,...))
local LSM = LibStub("LibSharedMedia-3.0")

local UnitPowerType = UnitPowerType
local UnitPower = UnitPower
local UnitPowerMax = UnitPowerMax
local BEAR_FORM = BEAR_FORM
local CAT_FORM = CAT_FORM
local SPELL_POWER_MANA = SPELL_POWER_MANA
local SPELL_POWER_RAGE = SPELL_POWER_RAGE
local SPELL_POWER_FOCUS = SPELL_POWER_FOCUS
local SPELL_POWER_ENERGY = SPELL_POWER_ENERGY
local SPELL_POWER_RUNIC_POWER = SPELL_POWER_RUNIC_POWER
local GetSpecialization = GetSpecialization
local GetShapeshiftFormID = GetShapeshiftFormID
local UnitHasVehicleUI = UnitHasVehicleUI
local UnitClass = UnitClass
local RAID_CLASS_COLORS = RAID_CLASS_COLORS
local GetSpellInfo = GetSpellInfo
local UnitAura = UnitAura
local unpack = unpack

function SN:SetupPower()
	if F.powermover ~= nil then
		F.powermover = nil
		F.powermover.value = nil
		F.container = nil
		F.power = nil
		F.power.value = nil
	end
	--The powerbar mover frame
	F.powermover = CreateFrame("Frame", nil, UIParent)
	F.powermover:SetPoint(unpack(SN.data.profile.frames.powerbar))
	F.powermover:SetSize(SN.data.profile.powerbar.width, SN.data.profile.powerbar.height)

	--The powerbar mover frame text
	F.powermover.value = F.powermover:CreateFontString(nil, "OVERLAY")
	F.powermover.value:SetPoint("CENTER", F.powermover)
	F.powermover.value:SetFontObject(GameFontNormal)
	F.powermover.value:SetJustifyH("CENTER")

	--The name to display when the powerbar is being moved
	F.powermover.name = "Power Bar"

	--The powerbar statusbar holder
	F.container = CreateFrame("Frame", nil, F.powermover)
	F.container:SetPoint("CENTER", F.powermover)
	F.container:SetSize(SN.data.profile.powerbar.width, SN.data.profile.powerbar.height)

	if (IsAddOnLoaded("ElvUI")) then
		SN:SNCreateBG(F.container)
	else
		F.container:SetBackdrop({
				bgFile = M.media.blankTex, 
				edgeFile = M.media.blankTex, 
				tile = false, tileSize = 0, edgeSize = 2, 
				insets = { left = 2, right = 2, top = 2, bottom = 2 }
				})
		F.container:SetBackdropColor(0.1, 0.1, 0.1, 1)
		F.container:SetBackdropBorderColor(0.1, 0.1, 0.1, 1)
		
	end

	--The powerbar statusbar
	F.power = CreateFrame("StatusBar", nil, F.container)
	F.power:SetPoint("CENTER", F.container)

	local powerType = UnitPowerType("player")

	if powerType > 5 then
		powerType = 5
	else
		powerType = powerType + 1
	end
	if (IsAddOnLoaded("ElvUI")) then
		F.power:SetSize(SN.data.profile.powerbar.width, SN.data.profile.powerbar.height)
	else
		F.power:SetSize(SN.data.profile.powerbar.width - 4, SN.data.profile.powerbar.height - 4)
	end
	F.power:SetStatusBarTexture(LSM:Fetch("statusbar", SN.data.profile.powerbar.texture))
	F.power:SetStatusBarColor(unpack(M.power[powerType]))
	local min = UnitPower("player")
	local max = UnitPowerMax("player")
	F.power:SetMinMaxValues(0, max)	
	F.power:SetValue(min)

	F.power:SetBackdrop({
		bgFile = M.media.blankTex,
	})
	F.power:SetBackdropColor(0.1, 0.1, 0.1, 1)
	F.power:SetAlpha(1)

	F.power:SetScript("OnEvent", function(self, event, ...)
		SN:PowerbarEvents(self, event, ...)
	end)
	
	local class, classFileName, classID = UnitClass("player")
	local colortmp = RAID_CLASS_COLORS[classFileName]
	local color = { colortmp.r, colortmp.g, colortmp.b }
	
	--The powerbar statusbar text
	F.power.value = F.power:CreateFontString(nil, "OVERLAY")
	F.power.value:SetPoint("CENTER", F.power)
	F.power.value:SetFont(LSM:Fetch("font", SN.data.profile.powerbar.font), SN.data.profile.powerbar.fontsize, "OUTLINE")
	F.power.value:SetJustifyH("CENTER")
	F.power.value:SetShadowOffset(1, -1)
	F.power.value:SetTextColor(unpack(color))
	F.power.value:SetText(string.format("%s / %s",  SN:ShortValue(min), SN:ShortValue(max)))

	--The powerbar events
	F.power:RegisterEvent("PLAYER_ENTERING_WORLD")
	F.power:RegisterEvent("UNIT_POWER_FREQUENT")
	F.power:RegisterEvent("UNIT_MAXPOWER")
	F.power:RegisterEvent("UNIT_DISPLAYPOWER")
	F.power:RegisterEvent("PLAYER_LEVEL_UP")

	if SN.data.profile.powerbar.enable then
		SN:PowerEnable()
		if (IsAddOnLoaded("ElvUI")) then
			SN:ElvOverride()
		end
	else
		SN:PowerDisable()
	end
end	

function SN:ShortValue(value)
	if value >= 1e6 then
		return ("%.1fm"):format(value / 1e6):gsub("%.?0+([km])$", "%1")
	elseif value >= 1e3 or value <= -1e3 then
		return ("%.1fk"):format(value / 1e3):gsub("%.?0+([km])$", "%1")
	else
		return value
	end
end

function SN:PowerbarEvents(self, event, ...)
	
	if event == "UNIT_POWER_FREQUENT" then
		local min = UnitPower("player")
		local max = UnitPowerMax("player")
		self:SetValue(min)
		self.value:SetText(string.format("%s / %s",  SN:ShortValue(min), SN:ShortValue(max)))
	end
	
	if event == "PLAYER_ENTERING_WORLD" then
		local powerType = UnitPowerType("player")

		if powerType > 5 then
			powerType = 5
		else
			powerType = powerType + 1
		end
		local min = UnitPower("player")
		local max = UnitPowerMax("player")
		self:SetStatusBarColor(unpack(M.power[powerType]))
		self:SetMinMaxValues(0, max)	
		self:SetValue(min)
		self.value:SetText(string.format("%s / %s",  SN:ShortValue(min), SN:ShortValue(max)))
	end

	if event == "UNIT_DISPLAYPOWER" then
		if UnitHasVehicleUI("player") then
			F.container:Hide()
		else
			if not F.container:IsShown() then
				F.container:Show()
			end
			local powerType = UnitPowerType("player")

			if powerType > 5 then
				powerType = 5
			else
				powerType = powerType + 1
			end
			local min = UnitPower("player")
			local max = UnitPowerMax("player")
			self:SetStatusBarColor(unpack(M.power[powerType]))
			self:SetMinMaxValues(0, max)	
			self:SetValue(min)
			self.value:SetText(string.format("%s / %s",  SN:ShortValue(min), SN:ShortValue(max)))
		end
	end

	if event == "PLAYER_LEVEL_UP" then
		local min = UnitPower("player")
		local max = UnitPowerMax("player")
		self:SetMinMaxValues(0, max)
		self:SetValue(min)
		self.value:SetText(string.format("%s / %s",  SN:ShortValue(min), SN:ShortValue(max)))
	end
end

function SN:PowerEnable()
	F.container:Show()
	F.power:RegisterEvent("UNIT_POWER_FREQUENT")
	F.power:RegisterEvent("UNIT_MAXPOWER")
	F.power:RegisterEvent("UNIT_DISPLAYPOWER")
end

function SN:PowerDisable()
	F.power:UnregisterEvent("UNIT_POWER_FREQUENT")
	F.power:UnregisterEvent("UNIT_MAXPOWER")
	F.power:UnregisterEvent("UNIT_DISPLAYPOWER")
	F.container:Hide()
end